import {Router} from "vue-router"

class RouteContext {

  private fullPath?: string
  private type?: string
  private direction?: string

  update(fullPath: string, type?: string, direction?: string): void {
    this.fullPath = fullPath
    this.type = type
    this.direction = direction
  }

  clearIfOtherPath(fullPath: string) {
    if (this.fullPath !== fullPath) {
      this.fullPath = undefined
      this.type = undefined
      this.direction = undefined
    }
  }

  // 현재 route 가 back/forward 버튼으로 이동된 것이라면 true 를 리턴한다.
  fromBackOrForward(): boolean {
    return !!(this.type && this.direction)
  }

  getType(): 'pop' | 'push' | undefined {
    // vue-router 의 NavigationType 이 export 타입이 아니라서, 이렇게 할 수밖에 없음.
    return (this.type === 'pop') ? 'pop' : (this.type === 'push') ? 'push' : undefined
  }

  getDirection(): 'back' | 'forward' | undefined {
    // vue-router 의 NavigationDirection 이 export 타입이 아니라서, 이렇게 할 수밖에 없음.
    return (this.direction === 'back') ? 'back' : (this.direction === 'forward') ? 'forward' : undefined
  }
}

export const routeContext = new RouteContext()

export const initializeRouteContext = (router: Router): void => {

  router.options.history.listen((to, from, info) => {
    // back/forward 로 인해 navigate 될 때마다 routeContext 를 update 한다.
    routeContext.update(to, String(info.type), String(info.direction))
  })

  router.afterEach((to) => {
    // 다른 곳으로 navigate 되는 경우, routeContext 를 clear 한다.
    routeContext.clearIfOtherPath(to.fullPath)
  })
}